/*
 * Copyright (c) 2009-2010 Jose-Luis Lisani <joseluis.lisani@uib.es>
 * All rights reserved.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef RGBCUBE_IPOL_H
#define RGBCUBE_IPOL_H

int count_RGBcolors(unsigned char *imR, unsigned char *imG, 
                    unsigned char *imB, 
                    int w, int h);

void PCAviews(unsigned char *imR, unsigned char *imG, unsigned char *imB, 
              int w, int h,
              unsigned char *refR, unsigned char *refG, unsigned char *refB, 
              int wref, int href,
              unsigned char *im12R, unsigned char *im12G, unsigned char *im12B,
              unsigned char *im13R, unsigned char *im13G, unsigned char *im13B,
              unsigned char *im23R, unsigned char *im23G, 
              unsigned char *im23B, 
              int wout, int hout);

void PCAviewsB(unsigned char *imR, unsigned char *imG, unsigned char *imB, 
               int w, int h,
               unsigned char *refR, unsigned char *refG, unsigned char *refB, 
               int wref, int href,
               unsigned char *im123R, unsigned char *im123G, 
               unsigned char *im123B, 
               int wout, int hout);

void PCAviews_density(unsigned char *imR, unsigned char *imG, 
                      unsigned char *imB, 
                      int w, int h,
                      unsigned char *refR, unsigned char *refG, 
                      unsigned char *refB, 
                      unsigned char *im12R, unsigned char *im12G, 
                      unsigned char *im12B,
                      unsigned char *im13R, unsigned char *im13G, 
                      unsigned char *im13B,
                      unsigned char *im23R, unsigned char *im23G, 
                      unsigned char *im23B, 
                      int wout, int hout,
                      int rdst, unsigned char cumdsty, unsigned char *Idsty);

void PCAviews_densityB(unsigned char *imR, unsigned char *imG, 
                       unsigned char *imB, 
                       int w, int h,
                       unsigned char *refR, unsigned char *refG, 
                       unsigned char *refB, 
                       unsigned char *im123R, unsigned char *im123G, 
                       unsigned char *im123B, 
                       int wout, int hout, int rdst, unsigned char cumdsty, 
                       unsigned char *Idsty);

void rmHighGradHue(unsigned char *imR, unsigned char *imG, unsigned char *imB, 
                   unsigned char *lowR, unsigned char *lowG, 
                   unsigned char *lowB,
                   unsigned char *highR, unsigned char *highG, 
                   unsigned char *highB, 
                   int w, int h, float thgrad, float thsat);

//type=1 -> 1D
//type=2 -> 2D
int filtercolor(unsigned char *inR, unsigned char *inG, unsigned char *inB,
                unsigned char *outR, unsigned char *outG, unsigned char *outB,
                int w, int h, int type, float eitmax, int r, int rsim);

//out=(in1 != 0)?(in1):(in2)
void mergeimages(unsigned char *in1R, unsigned char *in1G, unsigned char *in1B,
                 unsigned char *in2R, unsigned char *in2G, unsigned char *in2B,
                 unsigned char *outR, unsigned char *outG, unsigned char *outB,
                 int w, int h);

void densityImage(unsigned char *R, unsigned char *G, unsigned char *B, 
                  int w, int h, 
                  unsigned char *I, int rdst, unsigned char logscale);

int RGBviews_sequence(unsigned char *imR, unsigned char *imG, 
                      unsigned char *imB, 
                      int w, int h,
                      unsigned char ***viewsR, unsigned char ***viewsG, 
                      unsigned char ***viewsB, 
                      int wview, int hview, unsigned char displayDensity);

int RGBviews_sequence_params(const char *paramsfile);

void delete_views_sequence(unsigned char **viewR, unsigned char **viewG, 
                           unsigned char **viewB, 
                           int nviews);

void compute_RGB_PCA(unsigned char *R, unsigned char *G, unsigned char *B, 
                     int npoints, 
                     float &Rcm, float &Gcm, float &Bcm, float &vp1, 
                     float &vp2, float &vp3, 
                     float vc1[3], float vc2[3], float vc3[3]);

void projectRGBcube(unsigned char *R, unsigned char *G, unsigned char *B, 
                    int npoints, 
                    unsigned char *I,
                    unsigned char *Rout, unsigned char *Gout, 
                    unsigned char *Bout, 
                    int wout, int hout, 
                    float M[3], float u[3], float v[3]);

void projectRGBcubeB(unsigned char *R, unsigned char *G, unsigned char *B, 
                     int npoints, 
                     unsigned char *I, 
                     unsigned char *Rout, unsigned char *Gout, 
                     unsigned char *Bout, 
                     int wout, int hout, 
                     float M[3], float u[3], float v[3]);

void projectRGBdensities(unsigned char *R, unsigned char *G, unsigned char *B,
                         int npoints,
                         unsigned char *Rout, unsigned char *Gout, 
                         unsigned char *Bout, 
                         int wout, int hout, float M[3], float u[3], 
                         float v[3], 
                         float ***dsty, 
                         float &maxdsty, unsigned char logscale);

float diffRMSE(unsigned char *R1, unsigned char *G1, unsigned char *B1, 
               unsigned char *R2, unsigned char *G2, unsigned char *B2,
               int w, int h, unsigned char use_voxel, float &dmean);

void RGBcube2PLY(unsigned char *R, unsigned char *G, unsigned char *B, 
                 int npoints, 
                 unsigned char *I, const char *nameout);

void RGBcube2VRML2(unsigned char *R, unsigned char *G, unsigned char *B, 
                   int npoints, 
                   unsigned char *I, const char *nameout);

#endif
